/* refactoring0 */
package m.b;

import java.applet.*;
import java.awt.*;
import java.util.*;
import m.b.monsters.*;
import m.b.core.*;
import m.b.core.objs.*;
import m.b.gfx.*;
import m.gfx.*;
import m.util.*;

public class Battery extends GfxApplet implements Runnable {
  public World world;
  public Player player;
  public Screen screen;
  public Time77 time;
  public Logger logger;
  public Keyboard77 kbd;
  public Gfx gfx;

  /* Mouse clicked (for the very begining) */
  public boolean activated;

  /* to display controls at the first game screen */
  public boolean justStarted;

  /*
   * To not act but kbd for Intervals.nextLevelDelay time
   * When player just died=time.time
   */
  public double timeWhenLevelCompleted;

  /*
   * Is set in WorldLoader::load
   * After this time nothing but kbd acts Intervals.nextLevelDelay
   */
  public double timeWhenLevelLoaded;

  private Intro intro;
  private double lastFpsLog;
  public String name;
  AudioClip audio;
  
  public void initialize() {
    initInProgress=true;
    activated=false;
    justStarted=true;
    name="anonymous";
    enableEvents(AWTEvent.KEY_EVENT_MASK);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    time=new Time77();
    logger=new Logger(this);
    kbd=new Keyboard77(this, logger);
    new ConfigLoader(this);
    gfx=new Gfx(this);
    screen=new Screen(gfx.w, gfx.h);
    intro=new Intro(gfx);
    timeWhenLevelCompleted=0;
    timeWhenLevelLoaded=0;
    world=null;
    player=null;
    lastFpsLog=0;
    try {
      audio=getAudioClip(getDocumentBase(), "battery.mid");
      audio.loop();
    } catch(Exception e) {
      audio=null;
      exception(e);
    }
    System.gc();
    initialized=true;
  }

  public void stop() {
    super.stop();
    if (audio!=null) audio.stop();
    kbd.stop();
  }

  public synchronized void paint(Graphics g) {
    try {
      if (!initialized) {
        loadingScreen(g);
        if (!initInProgress) initialize();
      } else {
        if (thread != null) {
          gfx.M.newPixels();
          while (time.step()) {
            step();
          }
          time.nextFrame();
          if (intro != null) {
            intro.draw(time.time);
          } else {
            gfx.drawAll();
          }
          gfx.updateScreen();
        }
      }
      Thread.sleep(2);
    } catch(Exception e) {
      exception(e);
    }
  }

  private void loadingScreen(Graphics g) {
    g.setColor(Color.black);
    g.fillRect(0, 0, getWidth(), getHeight());
    g.setColor(new Color(128, 128, 64));
    g.setFont(new java.awt.Font("Fixedsys", java.awt.Font.BOLD, 10));
    g.drawString("BATTERY "+U77.ssprecision(Config.version, 2), 10, 20);
    g.drawString("http:"+P.bs+"btrgame.com", 10, 42);
    g.drawString("\2512008 M77", 10, 56);
    g.drawString("Loading.......", 10, 78);
    if (exception) {
      g.drawString(firstException.getClass().getName(), 10, 100);
      g.drawString(firstException.getMessage(), 10, 114);
    }
  }

  private void step() {
    if (time.time-lastFpsLog>Config.Replay.fpsLogPeriod) {
      lastFpsLog=time.time;
      logger.log("fps "+U77.sprecision(time.time)+" "+time.fps);
    }
    kbd.next();
    if (!activated && kbd.anyKey()) activated=true;
    if (time.time > Intro.duration) {
      if (time.time-timeWhenLevelLoaded>
          Config.Intervals.nextLevelDelay) {
        if ((time.time-timeWhenLevelCompleted>
            Config.Intervals.nextLevelDelay-(Time77.step*2)) &&
            (time.time-timeWhenLevelCompleted<
            Config.Intervals.nextLevelDelay) && (world!=null) &&
            (player.life>0)) {
          world.nextLevel();
        } else if (time.time-timeWhenLevelCompleted>
            Config.Intervals.nextLevelDelay) {
          step2();
        }
      }
    }
  }

  private void step2() {
    ifAfterIntro();
    if (activated) {
      if (justStarted && time.time-timeWhenLevelLoaded>
          Config.Intervals.nextLevelDelay) justStarted=false;
      screen.setCameraY(screen.cameraY()-Config.cameraSpeed*Time77.step, world);
      if (screen.camY()<Config.squareSize) {
        screen.setCameraY(screen.cameraY(), world);
        logger.log("lvlcompl "+U77.sprecision(time.time));
        timeWhenLevelCompleted=time.time;
      } else {
        if (!((time.time-timeWhenLevelCompleted >=
            Config.Intervals.nextLevelDelay) &&
            (time.time-timeWhenLevelLoaded >=
            Config.Intervals.nextLevelDelay))) {
          screen.setCameraY(screen.cameraY() + (Config.cameraSpeed*Time77.step),
              world);
        } else {
          actAll();
        }
      }
    }
  }

  private void ifAfterIntro() {
    if ((intro != null) || player.life==0) {
      intro=null;
      world=new World(gfx);
      world.activeObjs.add(player);
      world.addToMap(player);
      screen.setCameraY(screen.cameraY(), world);
    }
  }

  private void actAll() {
    for (Monster a: world.activeObjs) a.act();
    while (!world.objsToAdd.isEmpty()) {
      world.activeObjs.addAll(world.objsToAdd);
      ArrayList<Monster> objs=new ArrayList<Monster>(world.objsToAdd);
      world.objsToAdd.clear();
      for (Monster obj: objs) obj.act();
    }
    for (Monster a: world.objsToRemove) {
      world.activeObjs.remove(a);
      world.removeFromMap(a);
    }
    world.objsToRemove.clear();
    for (ChanSquare cs: world.notMonsters) {
      cs.act();
    }
  }

  public void exception(Exception e) {
    if (!exception) {
      try {
        logger.log(U77.toString(e));
      } catch(Exception ignored) {}
    }
    super.exception(e);
  }
}
