/* refactoring0 */
package m.b.core;

import java.awt.event.*;
import java.util.*;
import m.b.*;
import m.util.*;

public class Keyboard77 implements KeyListener, MouseListener {
  public ReplayClient replayClient;

  private Battery bat;
  private Logger logger;

  /*
   * Activated keys for this frame
   */
  private Set<String> keys;

  private Set<String> downKeys;
  private List<String> logStrings;

  public Keyboard77(Battery btr, Logger logger) {
    bat=btr;
    logStrings=new ArrayList<String>();
    keys=new HashSet<String>();
    downKeys=new HashSet<String>();
    bat.addKeyListener(this);
    bat.addMouseListener(this);
    replayClient=Config.Replay.active ? new ReplayClient(btr) : null;
    this.logger=logger;
  }

  synchronized public void stop() {
    replayClient.stop();
    logger.stop();
  }

  synchronized public void next() {
    logStrings=new ArrayList<String>();
    if (Config.Replay.active) {
      replayStep();
    }
    for (Iterator<String> it=keys.iterator(); it.hasNext();) {
      String key=it.next();
      if (!downKeys.contains(key)) {
        logger.log("kr "+U77.sprecision(bat.time.time)+" "+key);
      }
    }
    for (Iterator it=downKeys.iterator(); it.hasNext();) {
      String key=(String)it.next();
      if (!keys.contains(key)) {
        logger.log("kp "+U77.sprecision(bat.time.time)+" "+key);
      }
    }
    keys=new HashSet<String>();
    keys.addAll(downKeys);
  }

  synchronized public void keyTyped(KeyEvent ke) {
  }

  synchronized public void keyPressed(KeyEvent ke) {
    if (replayClient==null) keyPressed(keyEventToString(ke));
  }

  private void keyPressed(String key) {
    downKeys.add(key);
  }

  synchronized public boolean mouseClicked() {
    boolean res=keys.contains("mouse");
    downKeys.remove("mouse");
    return res;
  }

  synchronized public void keyReleased(KeyEvent ke) {
    if (replayClient==null) downKeys.remove(keyEventToString(ke));
  }

  synchronized public boolean up() {
    return keys.contains("up");
  }

  synchronized public boolean right() {
    return keys.contains("right");
  }

  synchronized public boolean down() {
    return keys.contains("down");
  }

  synchronized public boolean left() {
    return keys.contains("left");
  }

  synchronized public boolean ctrl() {
    return keys.contains("ctrl");
  }

  synchronized public boolean space() {
    return keys.contains("space");
  }

  synchronized public boolean anyKey() {
    return !keys.isEmpty();
  }

  synchronized public void mouseClicked(MouseEvent e) {
    if (replayClient==null) downKeys.add("mouse");
  }

  synchronized public void mouseEntered(MouseEvent e) {
  }

  synchronized public void mouseExited(MouseEvent e) {
  }

  synchronized public void mousePressed(MouseEvent e) {
    if (replayClient==null) downKeys.add("mouse");
  }

  synchronized public void mouseReleased(MouseEvent e) {
  }

  private static String keyEventToString(KeyEvent ke) {
    int code=ke.getKeyCode();
    switch (code) {
      case KeyEvent.VK_UP:
        return "up";
      case KeyEvent.VK_DOWN:
        return "down";
      case KeyEvent.VK_LEFT :
        return "left";
      case KeyEvent.VK_RIGHT:
        return "right";
      case KeyEvent.VK_CONTROL:
        return "ctrl";
      case KeyEvent.VK_SPACE:
        return "space";
    }
    return "another";      
  }

  private void replayStep() {
    int keyEventCount=replayClient.getEventCount();
    for (int i=0; i<keyEventCount; i++) {
      String key=replayClient.getKey();
      if (replayClient.pressed()) {
        emulate(key);
      } else {
        emulateRelease(key);
      }
    }
  }

  private void emulate(String key) {
    keyPressed(key);
  }

  private void emulateRelease(String key) {
    downKeys.remove(key);
  }
}
