package m.b.core;

import java.net.*;
import java.io.*;
import java.util.*;
import m.util.*;

public class ReplayConnection implements Runnable {
  private BufferedReader in;
  private PrintWriter out;
  private long id;
  private FileReader77 fr;
  private Map<String, String> params;
  private List<String> strs;

  public ReplayConnection(Socket sock, long id, FileReader77 fileReader) {
    try {
      this.id=id;
      this.fr=fileReader;
      in=new BufferedReader(new InputStreamReader(sock.getInputStream()));
      out=new PrintWriter(sock.getOutputStream(), true);
      initParams();
      P.p("replay connection "+id+" is created");
      new Thread(this).start();
    } catch(Exception ignored) {}
  }

  private final void initParams() {
    params=new HashMap<String, String>();
    strs=new ArrayList<String>();
    for (int i=0; i<fr.size(); i++) {
      String s=fr.read();
      if (s.startsWith(""+id)) {
        if ((s.indexOf(" kp ")!=-1) || (s.indexOf(" kr ")!=-1)) {
          strs.add(s);
        } else if (ifparam(s, "speed")) {
        } else if (ifparam(s, "maxSpeed")) {
        } else if (ifparam(s, "maxStrafeSpeed")) {
        } else if (ifparam(s, "minSpeed")) {
        } else if (ifparam(s, "heliMaxSpeed")) {
        } else if (ifparam(s, "levels")) {
        } else if (ifparam(s, "tankTurn")) {
        } else if (ifparam(s, "tankStays")) {
        } else if (ifparam(s, "bulletSpeed")) {
        } else if (ifparam(s, "shotInterval")) {
        } else if (ifparam(s, "nextLvlDelay")) {
        } else if (ifparam(s, "defaultA")) {
        } else if (ifparam(s, "cameraSpeed")) {
        } else if (ifparam(s, "tankSpeed")) {
        } else if (ifparam(s, "tankForPlayer")) {
        } else if (ifparam(s, "bulletGrowth"));
      }
    }
  }

  public void run() {
    try {
      int currentLine=0;
      double previousTime=0;

      while(true) {
        String str=in.readLine();
        if (str == null) {
          P.p("replay connection "+id+" terminated: maybe client is swithced off");
          return;
        } else if (str.indexOf("?")!=-1) {
          String param=str.substring(0, str.length()-1);
          out.println(params.get(param));
          continue;
        }
        double time=new Double(str).doubleValue();
        int requiredForNowStringCount=0;
        for (int i=currentLine; i<strs.size(); i++) {
          if (inTime(previousTime, time, strs.get(i))) {
            requiredForNowStringCount++;
          } else {
            break;
          }
        }
        out.println("" + requiredForNowStringCount);
        for (int i=0; i<requiredForNowStringCount; i++) {
          String s=(String)strs.get(currentLine);
          out.println(s.substring(s.lastIndexOf(' ')+1, s.length()));
          out.println(s.indexOf(" kp ") != -1 ? "true" : "false");
          currentLine++;
        }
        if (currentLine == strs.size()) {
          in.close();
          out.close();
          P.p("replay connection "+id+" terminated: end of log");
          return;
        }
        previousTime=time;
      }
    } catch(Exception e) {
      if ((!e.toString().startsWith("java.net.SocketException")) &&
          (!(e instanceof SocketTimeoutException))) {
        e.printStackTrace();
      }
      P.p("replay connection "+id+" terminated");
      return;
    }
  }

  private final boolean ifparam(String s, String param) {
    if (s.indexOf(param)!=-1) {
      params.put(param, afterLastSpace(s));
      return true;
    } else return false;
  }

  private final static String afterLastSpace(String s) {
    return s.substring(s.lastIndexOf(" ")+1);
  }

  private static boolean inTime(double prevTime, double time, String logString) {
    int index=logString.indexOf(' ');
    index=logString.indexOf(' ', index+1)+1;
    double logTime=
        new Double(logString.substring(index, logString.indexOf(' ', index))).doubleValue();
    return logTime>prevTime && logTime<=time;
  }
}
