/* refactoring0 */
package m.b.core;

import java.util.*;
import m.b.*;
import m.b.monsters.*;
import m.b.core.loader.*;
import m.b.core.objs.*;
import m.b.gfx.*;
import m.gfx.*;

public class World {
  public Gfx g;
  public Battery btr;
  public List<Monster> objsToAdd;
  public List<Monster> activeObjs;
  public List<Monster> objsToRemove;
  public List<Monster> objsToAddInTime;
  public List<ChanSquare> notMonsters;
  public int nextId;

  public int width;
  public int height;

  /*
   * Size of map elem equals Config.squareSize
   */
  private List<Drawable>[][] map;

  public int lvl;

  /*
   * (last+1) totalLvl == 1st lvl (not thinking about lvl mapping);
   */
  public int totalLvl;
 
  public World(Gfx gfx) {
    nextId=0;
    g=gfx;
    btr=gfx.btr;
    lvl=1;
    totalLvl=1;
    new WorldLoader(this);
  }

  public final int trueLevel() {
    return Config.levels.get(lvl-1);
  }

  public final void nextLevel() {
    lvl++;
    if (lvl==Config.levels.size()+1) lvl=1;
    totalLvl++;
    Screen screen=btr.screen;
    screen.init(screen.w, screen.h);
    new WorldLoader(this);
    Player player=btr.player;
    player.x=(double)width/2*Config.squareSize;
    player.y=(height-3)*Config.squareSize;
    screen.setCameraX(0);
    screen.setCameraY(height*Config.squareSize-screen.h, this);
    activeObjs.add(player);
    addToMap(player);
  }

  public void setMap(List<Drawable>[][] map) {
    this.map=map;
  }

  public java.util.List<Drawable> get(int x, int y) {
    if (x<0 || y<0 || x>=width || y>=height) {
      return new ArrayList<Drawable>();
    }
    return map[y][x];
  }

  public java.util.List<Drawable> get(Drawable d) {
    java.util.Set<Drawable> res=new HashSet<Drawable>();
    int yBorder=(int)(d.yEnd()-0.00001)/Config.squareSize+1;
    int xBorder=(int)(d.xEnd()-0.00001)/Config.squareSize+1;
    int xStart=(int)d.xStart()/Config.squareSize;
    for (int yy=(int)d.yStart()/Config.squareSize; yy<yBorder; yy++) {
      for (int xx=xStart; xx<xBorder; xx++) {
        res.addAll(get(xx, yy));
      }
    }
    return new ArrayList<Drawable>(res);
  }

  public java.util.List<Drawable> getNotChangeable(int xScreen, int yScreen,
      int xScreenBorder, int yScreenBorder) {
    Screen scr=btr.screen;
    int x=scr.camX()+xScreen;
    int y=scr.camY()+yScreen;
    int xBorder=scr.camX()+xScreenBorder;
    int yBorder=scr.camY()+yScreenBorder;
    java.util.Set<Drawable> res=new HashSet<Drawable>();
    xBorder=(xBorder-1)/Config.squareSize+1;
    yBorder=(yBorder-1)/Config.squareSize+1;
    int xStart=(int)(x)/Config.squareSize;
    for (int yy=(int)(y)/Config.squareSize; yy<yBorder; yy++) {
      for (int xx=xStart; xx<xBorder; xx++) {
        java.util.List<Drawable> drawables=get(xx, yy);
        for (Drawable d: drawables) {
          if (!(d instanceof Changeable)) {
            res.add(d);
          }
        }
      }
    }
    List<Drawable> list=new ArrayList<Drawable>(res);
    Collections.sort(list);
    return list;    
  }

  public java.util.List<Drawable> getChangeablesOnScreen() {
    java.util.Set<Drawable> res=new HashSet<Drawable>();
    Screen scr=btr.screen;
    int yBorder=scr.getYBorder();
    int xStart=scr.getStartX();
    int xBorder=scr.getXBorder();
    for (int yy=scr.getStartY(); yy<yBorder; yy++) {
      for (int xx=xStart; xx<xBorder; xx++) {
        java.util.List<Drawable> drawables=map[yy][xx];
        for (Drawable d: drawables) {
          if (d instanceof Changeable) {
            res.add(d);
          }
        }
      }
    }
    List<Drawable> list=new ArrayList<Drawable>(res);
    Collections.sort(list);
    return list;
  }

  public void removeFromMap(Drawable obj) {
    int yBorder=yMapBorder(obj);
    int xBorder=xMapBorder(obj);
    for (int y=yMapStart(obj); y<yBorder; y++) {
      for (int x=xMapStart(obj); x<xBorder; x++) {
        List<Drawable> drawables=get(x, y);
        for (int i=0; i<drawables.size(); i++) {
          if (drawables.get(i) == obj) {
            drawables.remove(i);
          }
        }
      }
    }
  }

  public void addToMap(Drawable obj) {
    int yBorder=yMapBorder(obj);
    int xBorder=xMapBorder(obj);
    for (int y=yMapStart(obj); y<yBorder; y++) {
      for (int x=xMapStart(obj); x<xBorder; x++) {
        List<Drawable> objs=get(x, y);
        if (!objs.contains(obj)) {
          objs.add(obj);
        }
      }
    }
  }

  public final void activateMonsters() {
    for (int i=0; i<objsToAddInTime.size(); i++) {
      Monster m=objsToAddInTime.get(i);
      if (m.yEnd()+Config.activationDistance>=btr.screen.cameraY()) {
        objsToAdd.add(m);
        addToMap(m);
        objsToAddInTime.remove(i--);
      }
    }
    int yStart=btr.screen.camY()/Config.squareSize;
    int yBorder=yStart+width;
    if (yBorder>height) yBorder=height;
    notMonsters.clear();
    for (int y=yStart; y<yBorder; y++) {
      for (int x=0; x<width; x++) {
        List<Drawable> list=map[y][x];
        for (Drawable d: list) {
          if (d instanceof ChanSquare && !notMonsters.contains(d)) {
            notMonsters.add((ChanSquare)d);
          }
        }
      }
    }
  }

  private int xMapStart(Drawable obj) {
    return (int)((obj.x-obj.hw))/Config.squareSize;
  }

  private int yMapStart(Drawable obj) {
    return (int)((obj.y-obj.hh))/Config.squareSize;
  }

  private int xMapBorder(Drawable obj) {
    return (int)((obj.x+obj.hw)-0.0001)/Config.squareSize+1;
  }

  private int yMapBorder(Drawable obj) {
    return (int)((obj.y+obj.hh)-0.0001)/Config.squareSize+1;
  }
}
