/* refactoring0 */
package m.b.core.loader;

import java.util.*;
import m.b.*;
import m.b.monsters.*;
import m.b.core.*;
import m.b.core.objs.*;
import m.b.gfx.*;
import m.b.monsters.items.*;
import m.gfx.*;
import m.util.*;

public class WorldLoader {
  private Battery btr;
  int lvl;
  private Gfx g;
  private World world;
  private int w;
  private int h;
  private List<Drawable>[][] map;

  /**
   * @param lvl - number of map
   */
  public WorldLoader(World world) {
    lvl=world.trueLevel();
    this.world=world;
    g=world.g;
    btr=g.btr;
    loadPart1();
    loadMain();
    specialMonsters();
    new BrickManager(map, world, g.getSprite("lvl"+lvl)).prepare();
    btr.timeWhenLevelLoaded=btr.time.time;
    btr.timeWhenLevelCompleted=0;
    btr.logger.log("lvl "+world.lvl+" "+U77.sprecision(btr.time.time));
    btr.logger.log("truelvl "+lvl);
    btr.logger.log("totlvl "+world.totalLvl);
    System.gc();
    U77.dropRandom();
  }

  private void loadPart1() {
    Sprite sprite=g.getSprite("lvl"+lvl);
    w=world.width=sprite.w;
    h=world.height=sprite.h;
    map=(List<Drawable>[][])new List<?>[h][w];
    world.setMap(map);
    world.objsToAdd=new ArrayList<Monster>();
    world.activeObjs=new ArrayList<Monster>();
    world.objsToRemove=new ArrayList<Monster>();
    world.objsToAddInTime=new ArrayList<Monster>();
    world.notMonsters = new ArrayList<ChanSquare>();
    for (int y=0; y<h; y++) {
      for (int x=0; x<w; x++) {
        map[y][x]=new ArrayList<Drawable>();
      }
    }
    if (btr.player==null || btr.player.life<=0) {
      g.btr.player=new Player(btr.kbd, world);
    }
  }

  private void loadMain() {
    int offset=0;
    Sprite sprite=g.getSprite("lvl"+lvl);
    for (int y=0; y<h; y++) {
      for (int x=0; x<w; x++) {
        int c=sprite.b[offset++];
        List<Drawable> list=map[y][x]=new ArrayList<Drawable>();
        if (c == 0xffff0000 || c==0xff808040) {
          list.add(new Square(g.getSprite("brickbig"), x, y, world, true, 1));
        } else if (c == 0xffc0c0c0) {
          list.add(new Square(g.getSprite("ground"), x, y, world, false, 0));
        } else if (c == 0xff808080) {
          list.add(new Square(g.getSprite("warfloor"), x, y, world, false, 0));
        } else if (c == 0xff8000ff) {
          Sprite spr=g.getSprite(Config.Decor.bck0?"bck0":
            (Config.Decor.grass?"grass":"carpet"));
          list.add(new Square(spr, x, y, world, false, 0));
        } else if (c==0xff008080) {
          throw new RuntimeException("corner should not be defined on image "+
              "map for a while x:"+x+" y:"+y);
        } else if (c==0xff004040) {
          list.add(new Square(g.getSprite("trap|"), x, y, world, false, 0));
        } else if (c==0xff008282) {
          list.add(new Square(g.getSprite("trap-"), x, y, world, false, 0));
        } else if (c==0xff804000) {
          int part=getLandingPart(x, y);
          if (part==0) {
            list.add(new LandingGround(x, y, world, false, 0));
          } else if (part==1) {
            list.add(getLG(map[y][x-1]));
          } else if (part==2) {
            list.add(getLG(map[y-1][x-1]));
          } else {
            list.add(getLG(map[y-1][x]));
          }
        } else if (c==0xffff0080) {
          list.add(background(x, y));
          firstaid(x, y);
        } else if (c==0xff008000) {
          Drawable sq=background(x, y);
          list.add(sq);
          heli(x, y);
        } else if (c==0xff00ffff) {
          list.add(background(x, y));
          enplane(x, y);
        } else if (c==0xff00ff80) {
          list.add(background(x, y));
          tank(x, y);
        } else {
          throw new RuntimeException("WorldLoader.load lvl (true number):"+lvl+" xy("+x+","+y+") color:"+C.string(c)+" should not be");
        }
      }
    }
  }

  private final int getLandingPart(int x, int y) {
    if (x==0 || !contains(map[y][x-1], "landingg")) {
      /*left*/
      if (y==0 || !contains(map[y-1][x], "landingg")) {
        /*up*/
        return 0;
      } else return 3;
    } else {
      /*right*/
      if (y==0 || !contains(map[y-1][x], "landingg")) {
        /*up*/
        return 1;
      } else return 2;
    }
  }

  private final LandingGround getLG(List<Drawable> list) {
    for (Drawable d: list) if (d instanceof LandingGround) {
      return (LandingGround)d;
    }
    throw new RuntimeException("have not found LG in this list");
  }

  private final void specialMonsters() {
//    if (lvl==1) {
//      heli(w, -2, 72);
//    }
  }

  private final Drawable background(int x, int y) {
    /* warfloor carpet ground*/
    int[] wcg=new int[6];
    for (int i: wcg) {
      i=0;
    }
    background(map[y-1][x-1], wcg);
    background(map[y-1][x], wcg);
    background(map[y-1][x+1], wcg);
    background(map[y][x-1], wcg);
    int max=U77.maxIndex(wcg);
    if (max==0) {
      return new Square(g.getSprite("warfloor"), x, y, world, false, 0);
    } else if (max==1) {
      return new Square(g.getSprite("carpet"), x, y, world, false, 0);
    } else if (max==2) {
      return new Square(g.getSprite("ground"), x, y, world, false, 0);
    } else if (max==3) {
      return new Square(g.getSprite("bck0"), x, y, world, false, 0);
    } else if (max==4) {
      return new Square(g.getSprite("grass"), x, y, world, false, 0);
    } else if (max==5) {
      return new Square(g.getSprite("trap|"), x, y, world, false, 0);
    } else {
      throw new RuntimeException("Should not be: unknown background");
    }
  }

  private static final boolean contains(List<Drawable> list, String starts) {
    for (Drawable d: list) {
      if (d instanceof Square) {
        Square sq=(Square)d;
        if (sq.sprite.name().startsWith(starts)) return true;
      }
    }
    return false;
  }

  private static final void background(List<Drawable> square, int[]wcg) {
    for (Drawable d: square) {
      if (d instanceof Square) {
        Square s=(Square)d;
        if (s.sprite.name().startsWith("warfloor")) {
          wcg[0]++;
        } else if (s.sprite.name().startsWith("carpet")) {
          wcg[1]++;
        } else if (s.sprite.name().startsWith("ground")) {
          wcg[2]++;
        } else if (s.sprite.name().startsWith("bck0")) {
          wcg[3]++;
        } else if (s.sprite.name().startsWith("grass")) {
          wcg[4]++;
        } else if (s.sprite.name().startsWith("trap")) {
          wcg[5]++;
        } else {
          P.p("unknown background "+s.sprite.name());
        }
      }
    }
  }

  private final void enplane(int x, int y) {
    EnPlane enplane=new EnPlane(squareCenter(x), squareCenter(y), world);
    world.objsToAddInTime.add(enplane);
  }

  private final void heli(int x, int y) {
    Heli heli=new Heli(squareCenter(x), squareCenter(y), world, 1);
    world.objsToAddInTime.add(heli);
  }

  private final void tank(int x, int y) {
    Tank tank=new Tank(squareCenter(x), squareCenter(y), world, 2);
    world.objsToAddInTime.add(tank);
  }

  private final void firstaid(int x, int y) {
    FirstAid fa=new FirstAid(squareCenter(x), squareCenter(y), world);
    world.objsToAddInTime.add(fa);
  }

  private static final double squareCenter(int square) {
    return (double)square*Config.squareSize+Config.hSquareSize;
  }
}
