package m.b.gfx;

import m.b.*;
import m.b.core.*;
import m.gfx.*;

import java.awt.image.*;
import java.util.*;
import java.awt.*;

public class Gfx {
  public MemoryImageSource M=null;
  public int w;
  public int h;
  public BufGfx b;
  public Battery btr;
  public m.gfx.Font font;
  public Console console;

  protected Map<String, Sprite> sprites;

  private Image screenImg;

  public Gfx(Battery battery) {
    btr=battery;
    w=btr.getSize().width;
    h=btr.getSize().height;
    b=new BufGfx(new int[w*h], w, h);
    font=new m.gfx.Font(0xffffffff, 0xff000000, b);
    console=new Console(font);
    M=new MemoryImageSource(w, h, new DirectColorModel(32, 0xff0000, 65280,
        255, 0), b.b, 0, w);
    M.setAnimated(true);
    M.setFullBufferUpdates(true);
    screenImg=btr.createImage(M);
    sprites=new HashMap<String, Sprite>();
    new SpriteLoader(this).load();
  }

  public final Sprite getSprite(String name) {
    Sprite s=sprites.get(name);
    if (s==null) {
      throw new RuntimeException("Gfx:getSprite ["+name+"] returns null");
    }
    return s;
  }

  protected final void putSprite(Sprite sprite) {
    sprites.put(sprite.name(), sprite);
  }

  public void drawAll() {
    drawNewBackAndForeGround();
    Screen screen=btr.player.screen;
    screen.getNewFrom(b.b);
    screen.copyTo(b.b);
    drawChangeables();
    if (btr.justStarted) {
      console.print("Use arrow keys to move the airplane.");
      console.print("Use Ctrl to shoot.");
      console.print("");
    }
    if (!btr.activated) {
      console.print("CLICK MOUSE TO START");
    } else {
      if (btr.time.time-btr.timeWhenLevelCompleted<
          Config.Intervals.nextLevelDelay) {
        if (btr.player.life==0) {
          console.print("GAME OVER");
        } else {
          console.print("LEVEL COMPLETED");
        }
      } else if (!btr.hasFocus()) {
        console.print("CLICK MOUSE");
      }
    }
    if (Config.debugMode) {
      String fps=""+btr.time.fps;
      font.p(fps, w-(fps.length()*m.gfx.Font.charWMin)-4, h-17);
    }
    drawStats(btr.player.life, btr.player.maxLife);
    console.print();
  }

  private void drawStats(double life, double max) {
    percentBar(btr.player.bullets, 0, 2, h-70, "ic_ammo");
    percentBar(life, max, 2, h-24, "ic_plane");
    percentBar(btr.player.diamonds, 0, 2, h-47, "ic_diam");
  }

  /**
   * @param max if max==0 - shadow bar
   */
  private final void percentBar(double units, double max, int x, int y,
      String icon) {
    b.rect(x, y, (int)100+4, 22, 0xff000000);
    b.rect(x+1, y+1, (int)100+2, 20, 0xffffffff);
    if (max>0) {
      int c=0xff006600;
      if ((icon.equals("ic_plane")) && (btr.player.afterDmg())) {
        c=0xffffffff;
      } else if (units/max<0.5) {
        if (units/max>=0.25) {
          c=0xffffff00;
        } else {
          c=0xffcc0000;
        }
      }
      b.filledRect(x+2, y+2, (int)units, 18, c);
      b.filledRect(x+2+(int)units, y+2, (int)max-(int)units, 18, 0xff000000);
    } else {
      b.rectShadow(x+2, y+2, 100, 18, 0.5);
    }
    String s=(max>0)?(""+(int)units+"/"+(int)max):(""+(int)units);
    font.pCenter(s, (int)100/2+4, y+10);
    b.drawTransp(getSprite(icon), 6, y+3);
  }

  /**
   * Now for scroll up only.
   */
  public void drawNewBackAndForeGround() {
    Screen scr=btr.screen;
    int shiftX=scr.camX()-scr.xGlobal();
    int shiftY=scr.camY()-scr.yGlobal();
    if (shiftX<=-w || shiftX >=w || shiftY<=-h || shiftY>=h) {
      java.util.List<Drawable> drawables =
          btr.world.getNotChangeable(0, 0, w, h);
      for (Drawable d: drawables) {
        d.draw(0, 0, w, h);
      }
    } else {
      if (shiftY < 0) {
        java.util.List<Drawable> drawables =
            btr.world.getNotChangeable(0, 0, w, -shiftY);
        for (Drawable d: drawables) {
          d.draw(0, 0, w, -shiftY);
        }
      } else if (shiftY==0) {
      } else {
      }
    }
  }

  private void drawChangeables() {
    java.util.List<Drawable> objs=btr.world.getChangeablesOnScreen();
    for (Drawable d: objs) {
      d.draw();
    }
  }

  public void updateScreen() {
    if (btr.exception && Config.debugMode) {
      font.p("ERROR", 3, 1);
    }
    btr.getGraphics().drawImage(screenImg, 0, 0, btr);
  }
}
