package m.b.gfx;

import java.util.*;
import m.b.core.*;
import m.gfx.*;

public class SpriteLoader {
  private Gfx g;

  protected SpriteLoader(Gfx gfx) {
    g=gfx;
  }
  
  protected final void load() {
    addSprite("lvl1", 17, 100);
    addSprite("lvl2", 17, 200);
    addSprite("lvl4", 17, 123);
    addSprite("lvl5", 17, 200);
    addSprite("battery", 375, 99);
    addSprite("ic_plane", 14, 16);
    addSprite("ic_diam", 16, 16);
    addSprite("ic_ammo", 16, 16);
    addSprite("copy", 87, 17);
    initDecor();
    initMonsters();
    initItems();
  }

  private final void initDecor() {
    addSprite4Dir("bordercorner", 30);
    addSprite4Dir("borderalmost", 30);
    addSprite4Dir("border", 30);
    addSpriteRot("bordertube");
    addSprite("bordered");
    addSprite("borderno");
    addSpriteRot("trap");
    Sprite warfloor=new Sprite(g.btr, "warfloor.png", 34, 34);
    warfloor.tile(2, 2);
    g.putSprite(warfloor);
    addSpriteUnitedSimmetricaly("landingground", 30);
    addSprite("brickbig");
    addSpriteRot("brick");
    addSpriteRot("bricko");
    addSpriteRot("brick1");
    addSpriteRot("brick1o");
    addSpriteRot("brick2");
    addSpriteRot("brick2o");
    addSpriteRot("brick3");
    addSpriteRot("brick3o");
    addSprite("s_brickbig");
    addSpriteRot("s_brick");
    addSpriteRot("s_bricko");
    addSpriteRot("s_brick1");
    addSpriteRot("s_brick1o");
    addSpriteRot("s_brick2");
    addSpriteRot("s_brick2o");
    addSpriteRot("s_brick3");
    addSpriteRot("s_brick3o");
    addSprite("ss_brickbig");
    addSpriteRot("ss_brick");
    addSpriteRot("ss_bricko");
    addSpriteRot("ss_brick1");
    addSpriteRot("ss_brick1o");
    addSpriteRot("ss_brick2");
    addSpriteRot("ss_brick2o");
    addSpriteRot("ss_brick3");
    addSpriteRot("ss_brick3o");
    addCyanBricks();
    addSprite("ground");
    addSprite("carpet");
    addSprite("bck0");
    addSprite("grass");
  }

  private final void addCyanBricks() {
    Set<String> sprites=(Set<String>)g.sprites.keySet();
    Set<Sprite> toAdd=new HashSet();
    for (String name: sprites) {
      if (name.startsWith("brick")) {
        Sprite s=g.getSprite(name);
        BufGfx b=new BufGfx(s, true);
        b.replaceColor(0xffff0000, 0xff808040);
        s=new Sprite("c_"+name, b);
        toAdd.add(s);
      }
    }
    for (Sprite sprite: toAdd) {
      g.sprites.put(sprite.name(), sprite);
    }
  }

  private final void addSpriteUnitedSimmetricaly(String name, int w) {
    BufGfx b=new BufGfx(w*2, w*2);
    Sprite sp=new Sprite(g.btr, name+".png", w, w);
    b.draw(sp, 0, 0);
    new BufGfx(sp).flipHorizontal();
    b.draw(sp, w, 0);
    new BufGfx(sp).flipVertical();
    b.draw(sp, w, w);
    new BufGfx(sp).flipHorizontal();
    b.draw(sp, 0, w);
    Sprite s=new Sprite(name, b);
    g.putSprite(s);
  }

  private final void initMonsters() {
    addSprite4Dir("bullet", 4, 9);
    addSprite("plane", 38, 47);
    addSprite("enplane", 51, 28);
    /*new Effects(new BufGfx(g.getSprite("enplane"))).inv();*/
    Sprite heli=new Sprite(g.btr, "heli.png", 49, 43);
    putSprite("heli", heli);
    Sprite heliLeft=new Sprite("heliLeft", heli, false);
    new BufGfx(heliLeft).flipHorizontal();
    putSprite("heliLeft", heliLeft);
    for (int i=0; i<7; i++) addSprite("blades"+i, 43, 43);
    addSprite4Dir("tank", 46);
    Sprite expl=addSprite("expl", 15);
  }

  private final void initItems() {
    addSprite("firstaid");
    addSprite("caterpillar", 16, 46);
    addSprite("diamond", 35, 35);
  }

  private final void addSprite(String name) {
    putSprite(name, new Sprite(g.btr, name+".png",
        Config.squareSize, Config.squareSize));
  }

  /**
   * Original, flipped horizontaly, flipped horizontaly & verticaly,
   *   flipped verticaly
   */
  private final void addSprite4Dir(String name, int size) {
    Sprite sprite=new Sprite(g.btr, name+".png", size, size);
    putSprite(name+"0", sprite);

    Sprite s=new Sprite(name+"1", sprite, false);
    new BufGfx(s).rot90();
    putSprite(name+"1", s);

    s=new Sprite(name+"2", s, false);
    BufGfx killme=new BufGfx(s);
    killme.rot90();
    putSprite(name+"2", s);

    s=new Sprite(name+"3", s, false);
    killme=new BufGfx(s);
    killme.rot90();
    putSprite(name+"3", s);
  }

  private final void addSprite4Dir(String name, int width, int height) {
    Sprite sprite=new Sprite(g.btr, name+".png", width, height);
    putSprite(name+"0", sprite);

    Sprite s=new Sprite(name+"1", sprite, false);
    BufGfx b=new BufGfx(s);
    b.rot90();
    s.setWH(b.w, b.h);
    putSprite(name+"1", s);

    s=new Sprite(name+"2", s, false);
    b=new BufGfx(s);
    b.rot90();
    s.setWH(b.w, b.h);
    putSprite(name+"2", s);

    s=new Sprite(name+"3", s, false);
    b=new BufGfx(s);
    b.rot90();
    s.setWH(b.w, b.h);
    putSprite(name+"3", s);
  }

  private final void addSpriteRot(String name) {
    Sprite s=new Sprite(g.btr, name+".png", Config.squareSize,
        Config.squareSize);
    putSprite(name+"|", s);
    s=new Sprite(name+"-", s, false);
    new BufGfx(s).rot90();
    putSprite(name+"-", s);
  }

  private final Sprite addSprite(String name, int size) {
    return addSprite(name, size, size);
  }

  private final Sprite addSprite(String name, int w, int h) {
    Sprite s=new Sprite(g.btr, name+".png", w, h);
    putSprite(name, s);
    return s;
  }

  private final void putSprite(String name, Sprite s) {
    s.setName(name);
    g.putSprite(s);
  }
}