package m.gfx;

import m.b.core.*;
import m.util.*;

public class Screen {
  public int w;
  public int h;
  private int[] b;
  private double cameraX;
  private double cameraY;
  private int camX;
  private int camY;

  /*
   * Previous camera coords
   */
  private double xGlobal;
  private double yGlobal;

  /*
   * Screen index
   */
  private int xStart;
  private int yStart;

  public Screen(int w, int h) {
    init(w, h);
  }

  public void init(int w, int h) {
    this.w=w;
    this.h=h;
    xStart=0;
    yStart=0;
    xGlobal=999999999;
    yGlobal=999999999;
    setCameraX(0);
    camY=0;
    cameraY=0;
    b=new int[h*w];
  }

  public double cameraX() {
    return cameraX;
  }

  public int camX() {
    return camX;
  }

  public int xGlobal() {
    return (int)xGlobal;
  }

  public void setCameraX(double v) {
    cameraX=v;
    camX=(int)cameraX;
  }

  public double cameraY() {
    return cameraY;
  }

  public int camY() {
    return camY;
  }

  public int yGlobal() {
    return (int)yGlobal;
  }

  public void setCameraY(double v, World w) {
    cameraY=v;
    camY=(int)cameraY;
    w.activateMonsters();
  }

  /**
   *   cameraX(xStart)
   *       V
   *  |o|o|x|_|o|o|   shiftX=2, shiftY=1
   *  |o|o|x|_|o|o|
   *  |x|x|X|x|x|x|<cameraY(yStart)
   *  |o|o|x|_|O|o|
   *  |o|o|x|_|o|o|
   */
  public void getNewFrom(int[] p) {
    int shiftX=camX-(int)xGlobal;
    int shiftY=camY-(int)yGlobal;
    xGlobal=cameraX;
    yGlobal=cameraY;
    if (shiftX<=-w || shiftX >=w || shiftY<=-h || shiftY>=h) {
      xStart=0;
      yStart=0;
      System.arraycopy(p, 0, b, 0, h*w);
      return;
    }
    xStart=U77.rem(xStart+shiftX, w);
    yStart=U77.rem(yStart+shiftY, h);
    if (shiftY < 0) {
      getNew(-shiftY, p);
    } else if (shiftY==0) {
    } else {
    }
  }

  /**
   *  | | |x|_|_|_|
   *  |_|_|x|_|_|_|
   *  |x|x|X|x|x|x|-xStart, yStart
   *  |_|_|x|_|_|_|
   *  |_|_|x|_|_|_|
   */
  public void copyTo(int[] p) {
    int offset=0;
    for (int y=yStart; y<h; y++) {
      int pix=y*w+xStart;
      System.arraycopy(b, pix, p, offset, w-xStart);
      pix -= xStart;
      System.arraycopy(b, pix, p, offset+xStart, w-xStart);
      offset += w;
    }
    for (int y=0; y<yStart; y++) {
      int pix=y*w+xStart;
      System.arraycopy(b, pix, p, offset, w-xStart);
      pix -= xStart;
      System.arraycopy(b, pix, p, offset+xStart, w-xStart);
      offset += w;
    }
  }

  /**
   * From yStart height lines down
   */
  private void getNew(int height, int[] p) {
    int offset=0;
    int yScreenBorder=yStart+height;
    if (yScreenBorder>h) yScreenBorder=h;
    for (int y=yStart; y<yScreenBorder; y++) {
      int pix=y*w+xStart;
      System.arraycopy(p, offset, b, pix, w-xStart);
      pix -= xStart;
      System.arraycopy(p, offset+xStart, b, pix, xStart);
      offset += w;
    }
    if (yScreenBorder<yStart+height) {
      yScreenBorder=yStart+height-h;
      for (int y=0; y<yScreenBorder; y++) {
        int pix=y*w+xStart;
        System.arraycopy(p, offset, b, pix, w-xStart);
        pix -= xStart;
        System.arraycopy(p, offset+xStart, b, pix, xStart);
        offset += w;
      }
    }
  }

  public int xEnd() {
    return camX+w-1;
  }

  public double yEnd() {
    return camY+h-1;
  }

  public int getStartX() {
    return (int)(cameraX/Config.squareSize);
  }

  public int getStartY() {
    return (int)(cameraY/Config.squareSize);
  }

  public int getXBorder() {
    return (int)((cameraX+w-1)/Config.squareSize)+1;
  }

  public int getYBorder() {
    return (int)((cameraY+h-1)/Config.squareSize)+1;
  }
}
