package m.tools.javastats77;

import de.java2html.Java2Html;
import java.io.*;
import java.util.*;
import m.util.*;

public class JavaStats77 {
  private final static String version="1.03";

  private final static String white="<font color=\"ffffff\">";

  protected static final boolean outputProgress=true;

  private static String res;
  private static String hrefDir;
  private static String outputDir;
  private static String add;
  private static String addTop;
  private static double total;

  /**
   * args[0] - path to the folder with .java classes. Example: "src".
   * args[1] - path to the file where html code for classes chart will be saved.
   *   Example: "result.txt"
   * args[2] - relative path for html a href attrs referencing to the htmled
   *   classes from class chart. If the path is "" then the argument should
   *   be "-"
   * args[3] - path to the output directory where htmled .java files will be
   *   saved. Example: "javahtml". Optional parameter. By default output
   *   directory is "output".
   * args[4] - add to each java 2 html file end. Optional arg.
   */
  public static void main(String[] args) {
    try {
      if (args.length > 2) {
        hrefDir="";
        outputDir="output/";
        add="Generated with <a href=\"http://javastats77.btrgame.com\">JavaStats77 "+version+"</a>";
        addTop=
"<script type=\"text/javascript\"><!--\n"+
"google_ad_client = \"pub-6741247462366009\";\n"+
"/* 468x60, создано 20.05.08 */\n"+
"google_ad_slot = \"0919203059\";\n"+
"google_ad_width = 468;\n"+
"google_ad_height = 60;\n"+
"google_cpa_choice = \"\"; // on file\n"+
"//-->\n"+
"</script>\n"+
"<script type=\"text/javascript\"\n"+
"src=\"http://pagead2.googlesyndication.com/pagead/show_ads.js\">\n"+
"</script>\n"+
"<p>";

        if (args.length>2 && !args[2].equals("-")) hrefDir=args[2];
        if (args.length>3) outputDir=args[3]+"/";
        if (args.length>4) add=args[4];
        Node root=new Node(null, new File(args[0]),0,new File(args[0]).getName()+"/");
        root.count();
        total=root.sum+root.childSum;
        res =
"<html>\n"+
"  <head>\n"+
"    <link rel=\"stylesheet\" href=\"style.css\" type=\"text/css\">\n"+
"  </head>\n"+
"  <body>\n"+
"<center>\n"+
"<font size=\"2\">Generated with <a href=\"http:"+P.bs+"btrgame.com/dev/javastats77/\">JavaStats77 "+version+"</a>"+
    " on "+Date77.datetime()+"</font>\n"+
"<p>\n";
        for (int lvl=0; lvl<100; lvl++) {
          if (!print(root, true, lvl)) break;
        }
        res +=
"</center>\n"+
"  </body>\n"+
"</html>\n";

        File77.create(args[1], res);
      } else {
        P.p("Call JavaStats77 with at least 3 arguments.");
        P.p("Example:");
        P.p("java -cp .;java2html.jar m.tools.javastats77.JavaStats77 src result.txt - output");
        P.p("  arg1 - path to the folder with .java classes. "+
            "Example: \"src\".");
        P.p("  arg2 - path to the file where html code for "+
            "classes chart will be saved.\n"+
            "    Example: \"result.txt\"");
        P.p("  arg3 - relative path for html a href attrs "+
            "referencing to the htmled\n"+
            "    classes from class chart. If the path is \"\" then the argument\n"+
            "    should be \"-\"");
        P.p("  arg4 - path to the output directory where htmled\n"+
            "    .java files will be saved. Example: \"javahtml\".\n"+
            "    It is an optional parameter. By default output directory is \"output\".");
        P.p("  args5 - add to each java 2 html file end. Optional arg.");
      }
    } catch(Exception e) {
      P.p("Error:");
      e.printStackTrace();
    }
  }

  private static boolean print(Node n, boolean root, int lvl) {
    P.pp(".");
    Collections.sort(n.childs);
    if (root) {
      if (lvl>n.lvl) {
        if (n.childs.isEmpty()) {
          return false;
        } else {
          boolean printed=false;
          for (Node child: n.childs) {
            if (child.f.isDirectory()) {
              if (print(child, true, lvl)) printed=true;
            }
          }
          return printed;
        }
      } else if (lvl<n.lvl) return false;
    } else if (n.lvl!=lvl) return false;
    if (root) {
      res+=
"<table border=\"0\" cellspacing=\"1\" cellpadding=\"0\" width=\"300\">\n"+
"  <tr><td background=\""+pipe((double)(n.sum+n.childSum)/total, true)+"\"><table width=\"100%\"><tr><td align=\"left\">"+
    "<a name=\""+n.getPackageName()+"\">"+white+n.getPackageName()+"</font></a></td><td align=\"right\">"+
    U77.sprecision((double)(n.sum+n.childSum)/1024, 2)+" KB</td></tr></table></td></tr>\n";
      for (Node child: n.childs) print(child, false, lvl+1);
      res+=
"</table>\n"+
"<p>\n";
    } else {
      double parentSum=(double)(n.parent.sum+n.parent.childSum);
      double sum=(double)(n.sum+n.childSum);
      res+=
"  <tr><td background=\""+pipe(sum/parentSum, false)+"\"><table width=\"100%\"><tr><td align=\"left\">"+
    "&nbsp;&nbsp;&nbsp;<a href=\""+href(n)+"\">"+(n.f.isDirectory()?white:"")+n.f.getName()+(n.f.isDirectory()?"</font>":"")+"<a></td><td align=\"right\">"+
    U77.sprecision(sum/parentSum*100, 2)+"%</td></tr></table></td></tr>\n";
    }
    return true;
  }

  private static String href(Node n) {
    if (n.f.isFile()) {
      String name=n.f.getName();
      String ref=n.path+name.substring(0, name.indexOf('.'))+
          ".html";
      String content=File77.getContent(n.f.getAbsolutePath());
      File77.create(outputDir+ref, addTop+Java2Html.convertToHtml(content)+add);
      return (hrefDir.equals("") ? "" : hrefDir+"/")+ref;
    } else {
      return "#"+n.getPackageName();
    }
  }

  /**
   * @param length 0<=length<=1
   */
  private static String pipe(double length, boolean root) {
    int size=(int)(299*length+0.5);
    if (size>299) size=299;
    return "pipes/"+(root?"g":"r")+size+".png";
  }
}
