package m.tools.nn77;

import java.io.*;
import java.util.*;
import m.util.*;

/**
 * Version 1.002
 */
public class Net77 implements Serializable {
  private static double defaultLearningRate=0.1;
  private static boolean defaultProgressOutput=true;

  public double learningRate;
  public boolean progressOutput;
  public long learningTime;
  public List<NNPattern> patterns;
  public Printer printer;
  public Advanced a;

  protected List<Neuron> inputs;
  protected List<Neuron> outputs;
  protected List<List<Neuron>> layers;

  public Net77(int inpCount, int outCount, int layerCount, int layerLength) {
    learningRate=defaultLearningRate;
    progressOutput=defaultProgressOutput;
    learningTime=0;
    patterns=new ArrayList<NNPattern>();
    printer=new Printer(this);
    buildNet(inpCount, outCount, layerCount, layerLength);
  }

  public void rate(double... inps) {
    drop();
    int i=0;
    for (double d: inps) inputs.get(i++).value=d;
    for (Neuron n: outputs) n.calc();
    //hereP.p()
  }

  public void learn(long iterationCount) {
    long time=System.currentTimeMillis();
    P.p("Start to learn "+iterationCount+" iterations");
    long prevProc=-1;
    for (long i=0; i<iterationCount; i++) {
      if (i*100/iterationCount>prevProc) {
        prevProc=i*100/iterationCount;
        P.p(""+prevProc+"%");
      }
      for (NNPattern p: patterns) learn(p);
    }
    P.p(""+iterationCount+" were learned for "+
        (((double)System.currentTimeMillis()-time)/1000)+" sec");
  }

  /**
   * One iteration. Possibly the test method.
   */
  public void learn(NNPattern desired) {
System.out.println("before:"+Str77.toString(desired.outputs));
printer.print();
    dropAksonsChanged();
    rate(desired.inputs);
    int i=0;
    for (double d: desired.outputs) {
      outputs.get(i).error(d);
    }
//System.out.println("after:"+Str77.toString(desired.outputs));
//printer.print();
  }

  protected static double backfunc(double d) {
    return d*(1-d);
  }
  
  public static double func(double d) {
    return 1.0/(1+Math.exp(-d));
  }

  private void buildNet(int inpCount, int outCount, int layerCount,
      int layerLength) {
    inputs=new ArrayList<Neuron>();
    for (int i=0; i<inpCount; i++) {
      inputs.add(new Neuron());
    }
    outputs=new ArrayList<Neuron>();
    for (int i=0; i<outCount; i++) {
      outputs.add(new Neuron(true, this));
    }
    layers=new ArrayList<List<Neuron>>();
    for (int i=0; i<layerCount; i++) {
      List<Neuron> layer=new ArrayList<Neuron>();
      layers.add(layer);
      for (int j=0; j<layerLength; j++) {
        layer.add(new Neuron(true, this));
      }
    }
    connectLayers(inputs, layers.get(0));
    connectLayers(layers.get(layers.size()-1), outputs);
    for (int i=0; i<layers.size()-1; i++) {
      connectLayers(layers.get(i), layers.get(i+1));
    }
    drop();
  }


  private void connectLayers(List<Neuron> inps, List<Neuron> outs) {
    for (Neuron i: inps) for (Neuron o: outs) new Akson(i, o, this);
  }

  /**
   * Except biases. Weights of aksons doesn't change.
   */
  public void drop() {
    Set<Neuron> dropped=new HashSet<Neuron>();
    for (Neuron n: outputs) {
      n.dropRec(inputs, dropped);
    }
  }

  public void dropAksonsChanged() {
    Set<Akson> dropped=new HashSet<Akson>();
    for (Neuron n: outputs) {
      for (Akson a: n.inputs) a.dropChangedRec(inputs, dropped);
    }
  }

  public void dropWeights() {
    Set<Akson> dropped=new HashSet<Akson>();
    for (Neuron n: outputs) {
      n.dropWeightsRec(dropped);
    }
  }
}
