package m.tools.nn77;

import java.util.*;

public class Neuron {
  private static long nextId=0;

  protected long id;
  protected double value;
  protected double error;
  protected List<Akson> inputs;
  protected List<Akson> outputs;

  protected boolean calced;
  protected boolean errorCalced;

  public Neuron() {
    id=nextId++;
    value=0;
    error=0;
    calced=false;
    errorCalced=false;
    inputs=new ArrayList<Akson>();
    outputs=new ArrayList<Akson>();
  }

  public Neuron(boolean hasBias, Net77 net) {
    this();
    if (hasBias) {
      Neuron n=new Neuron();
      n.value=1;
      new Akson(n, this, net);
    }
  }

  public double calc() {
    if (!calced && !inputs.isEmpty()) {
      calced=true;
      double sum=0;
      for (Akson a: inputs) {
        sum+=a.input.calc()*a.weight;
      }
      value=Net77.func(sum);
    }
    return value;
  }

  /**
   * For output neurons.
   */
  public void error(double desired) {
    error=(desired-value)*Net77.backfunc(value);
    errorCalced=true;
    for (Akson a: inputs) {
      a.weight();
      a.input.error();
    }
  }

  /**
   * For not output neurons.
   */
  public double error() {
    if (errorCalced) return error;
    if (inputs.isEmpty()) return 0;
    double sum=0;
    for (Akson a: outputs) {
      sum+=a.output.error()*a.weight();
    }
    error=Net77.backfunc(value)*sum;
    errorCalced=true;
    for (Akson a: inputs) a.weight();
    return error;
  }

  public void drop() {
    value=0;
    error=0;
    calced=false;
    errorCalced=false;
  }

  /**
   * @param inputs Drop except biases.
   */
  public void dropRec(List<Neuron> netInputs, Set<Neuron> dropped) {
    if (!(inputs.isEmpty() || netInputs.contains(this)
        || dropped.contains(this))) {
      drop();
      dropped.add(this);
      for (Akson a: inputs) {
        a.input.dropRec(netInputs, dropped);
      }
    }
  }

  public void dropWeightsRec(Set<Akson> dropped) {
    for (Akson a: inputs) {
      if (dropped.contains(a)) continue;
      a.weight=0;
      dropped.add(a);
      a.input.dropWeightsRec(dropped);
    }
  }

  public String toString() {
    return "N["+id+"] v:"+value+" err:"+error;
  }
}
