package m.tools.nn77;

import java.util.*;
import m.util.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;

public class Printer {
  private String where;
  
  private Net77 net;
  private Set<Akson> printedAksons;
  private Set<Neuron> printedNeurons;
  private Map<Neuron, Pair> neurons;

  protected Printer(Net77 net) {
    where="C:\\Docume~1\\Administrator\\Desktop\\";
    this.net=net;
    printedAksons=new HashSet<Akson>();
    printedNeurons=new HashSet<Neuron>();
    neurons=new HashMap<Neuron, Pair>();
  }

  public void outputs() {
    P.p("\n********** PRINT OUTPUTS **********");
    for (Neuron n: net.outputs) {
      P.p(""+n);
    }
    P.p("******************************\n");
  }

  public void print() {
    BufferedImage img=new BufferedImage(1000, 1000, BufferedImage.TYPE_INT_RGB);
    Graphics g=img.createGraphics();
    g.setColor(Color.BLACK);
    g.fillRect(0, 0, 1000, 1000);
    for (Neuron n: net.inputs) {
      draw(n, g);
    }
    for (Neuron n: net.outputs) {
      draw(n, g);
    }
    for (java.util.List<Neuron> layer: net.layers) {
      for (Neuron n: layer) {
        draw(n, g);
      }
    }

    try {
      String filename=where+"nn.png";
      ImageIO.write(img, "png", new File(filename));
      Runtime r=Runtime.getRuntime();
      Process p= null;
      p=r.exec("mspaint.exe "+filename) ;
    } catch(Exception e) {
      e.printStackTrace();
    }

    //here
    printedAksons.clear();
    printedNeurons.clear();
    P.p("\n\n\n********** PRINT NET **********");
    print(net.inputs.get(0));
    P.p("\nneuron count:"+printedNeurons.size());
    P.p("******************************\n");
  }

  private void draw(Neuron n, Graphics g) {
    P.p("drawn at ("+getX(n)+","+getY(n)+")");
    draw(n, g, getX(n), getY(n));
    if (!net.inputs.contains(n)) {
      for (Akson a: n.inputs) {
        Neuron neur=a.input;
        //todo something??
      }
    }
  }

  private int getX(Neuron n) {
    if (net.inputs.contains(n)) {
      return net.inputs.indexOf(n)*170+20;
    }
    if (net.outputs.contains(n)) {
      return net.outputs.indexOf(n)*170+20;
    }
    for (java.util.List<Neuron> layer: net.layers) {
      int i=0;
      for (Neuron neur: layer) {
        if (neur==n) {
          return i*170+20;
        }
        i++;
      }
    }
    throw new RuntimeException("biasi eshe ne sdelal");
  }

  private int getY(Neuron n) {
    if (net.inputs.contains(n)) {
      return 880;
    }
    if (net.outputs.contains(n)) {
      return 20;
    }
    int i=1;
    for (java.util.List<Neuron> layer: net.layers) {
      i++;
      if (layer.contains(n)) return -170*(i)+1020;
    }
    throw new RuntimeException("biasi eshe ne sdelal");
  }

  private void draw(Neuron n, Graphics g, int x, int y) {
    neurons.put(n, new Pair(x, y));
    g.setColor(Color.GRAY);
    g.fillOval(x, y, 100, 100);
    g.setColor(Color.WHITE);
    x+=13;
    y+=20;
    g.drawString((n.inputs.isEmpty()?"inp ":"")+"id: "+n.id, x, y+12);
    g.drawString("v: "+n.value, x, y+24);
    g.drawString("err: "+n.error, x, y+36);
    g.drawString("calc: "+n.calced, x, y+48);
    g.drawString("errcalc: "+n.errorCalced, x, y+60);
  }

  private void print(Neuron n) {
    printedNeurons.add(n);
    String inpout="";
    if (n.inputs.isEmpty()) {
      inpout="inp ";
    } else if (n.outputs.isEmpty()) {
      inpout="out ";
    }
    P.p(inpout+n);
    for (Akson akson: n.inputs) {
      if (!printedAksons.contains(akson)) {
        print(akson);
      }
    }
    for (Akson akson: n.outputs) {
      if (!printedAksons.contains(akson)) {
        print(akson);
      }
    }
  }

  private void print(Akson a) {
    printedAksons.add(a);
    P.p("["+a.input.id+"]- "+a.weight+" -["+a.output.id+"]");
    if (!printedNeurons.contains(a.input)) {
      print(a.input);
    }
    if (!printedNeurons.contains(a.output)) {
      print(a.output);
    }
  }
}
