package m.tools.sitestats77;

import java.io.*;
import java.util.*;
import javax.servlet.http.*;
import m.util.*;

/**
 * @version 1.03
 */
public class SiteStats implements Serializable {
  private static final String saveDir =
      "C:/tomcat/webapps/ROOT/stats";
  private static final String fileName="sitestats.serialized";

  private static long hitsTillAutosave;

  public static SiteStats stats=null;

  protected Map<String, URLStats> urls;
  protected Map<String, UserStats> users;

  protected SiteStats() {
    urls=new HashMap<String, URLStats>();
    users=new HashMap<String, UserStats>();
    hitsTillAutosave=50;
  }

  public static synchronized void init() {
    try {
      if (stats == null) {
        stats=(SiteStats)Serialization.deserialize(saveDir+"/"+fileName);
      }
    } catch(Exception e) {
      P.p("Error deserializing SiteStats:\n"+U77.toString(e));
      stats=new SiteStats();
    }
    hitsTillAutosave=100;
  }

  public static synchronized String save() {
    if (stats == null) {
      SiteStats.init();
      P.p("SiteStats77: nothing to save");
      return "SiteStats77: nothing to save";
    } else {
      Serialization.serialize(stats, saveDir+"/"+fileName);
      P.p("SiteStats77: saved");
      return "SiteStats77: saved";
    }
  }

  public synchronized void hit(HttpServletRequest req) {
    String ip=req.getRemoteAddr();
    String url=getMyUrl(req);
    Hit hit=new Hit(null, null, req);
    URLStats urlStats=null;
    if (urls.containsKey(url)) {
      urlStats=urls.get(url);
    } else {
      urlStats=new URLStats(url);
      urls.put(url, urlStats);
    }
    urlStats.hits.add(hit);
    hit.url=urlStats;

    UserStats userStats=null;
    if (users.containsKey(ip)) {
      userStats=users.get(ip);
    } else {
      userStats=new UserStats(ip);
      users.put(ip, userStats);
    }
    userStats.hits.add(hit);
    hit.user=userStats;
    urlStats.users.add(userStats);
    if (--hitsTillAutosave==0) {
      hitsTillAutosave=100;
      save();
    }
  }

  public synchronized int getHostCount(String url) {
    return urls.get(url).users.size();
  }

  public synchronized int getHitCount(String url) {
    return urls.get(url).hits.size();
  }

  public synchronized final List<Hit> allHits() {
    List<Hit> hits=new ArrayList<Hit>();
    for (URLStats url: urls.values()) {
      hits.addAll(url.hits);
    }
    return hits;
  }


  public synchronized String generate(String dir) {
    String saved=save();
    if (dir==null || dir.equals("")) {
      dir="C:/tomcat/webapps/ROOT/stats";
    }
    List<Hit> hits=new ArrayList<Hit>();
    for (UserStats user: users.values()) {
      hits.addAll(user.hits);
    }
    Collections.sort(hits);
    String res =
"<html>\n"+
"  <head>\n"+
"    <title>Site Statistics by SiteStats77 "+Date77.datetime()+"</title>\n"+
"    <link rel=\"stylesheet\" href=\"style.css\" type=\"text/css\">\n"+
"  </head>\n"+
"  <body>\n"+
"<b>"+Date77.datetime()+" generated by SiteStats77 1.03</b><br>\n"+
"<a href=\"by_days.html\">by_days</a><p>\n"+        
"<table border=\"1\">\n";
    int day=-1;
    int hitNum=-1;
    int hitTotal=hits.size();
    int prevProc=-1;
    for (Hit hit: hits) {
      hitNum++;
      if (hitNum*100/hitTotal>prevProc) {
        prevProc=hitNum*100/hitTotal;
        P.p("SiteStats.stats.generate() hits: "+prevProc+"%");
      }
      if (day<Date77.dayOfYear(hit.time)) {
        day=Date77.dayOfYear(hit.time);
        res += "<tr><td colspan=\"7\"><b>"+Date77.date(hit.time)+"</b></td></tr>\n"+
            "<tr><td><b>time</b></td><td><b>ip</b></td><td><b>url</b></td>"+
            "<td><b>referer</b></td><td><b>params</b></td>"+
            "<td><b>browser</b></td><td><b>locale</b></td></tr>\n";
      }
      res += "<tr><td>"+Date77.time(hit.time)+"</td><td>"+hit.user.ip+"</td><td>"+hit.url.url+
          "</td><td>"+getReferer(hit.referer)+"</td><td>"+(hit.params==null?"":hit.params)+
          "</td><td>"+getBrowser(hit.browser)+"</td><td>"+hit.locale+"</td></tr>\n";
    }
    createByDays(dir);
    File77.create(dir+"/stats.html", res+"</table><p>\n"+
        "  </body>\n</html>\n");
    return saved+"<br>\n<a href=\"stats/stats.html\">ready</a>";
  }

  private synchronized final void createByDays(String dir) {
    String res=
"<html>\n"+
"  <head>\n"+
"    <title>By Days "+Date77.datetime()+"</title>\n"+
"    <link rel=\"stylesheet\" href=\"style.css\" type=\"text/css\">\n"+
"  </head>\n"+
"  <body>\n"+
"<b>By Days "+Date77.datetime()+"</b><br><p>\n"+
"<table border=\"1\">\n";
    List<Hit> hits=allHits();
    Collections.sort(hits, new Comparator() {
      public int compare(Object o1, Object o2) {
        Hit h1=(Hit)o1;
        Hit h2=(Hit)o2;
        return (int)(h1.time-h2.time);
      }
      public boolean equals(Object o) {
        return false;
      }
    });
    int day=0;
    String th="</b></td><td><b>";
    String td="</td><td>";
    Set<String> ips=null;
    for (Hit hit: hits) {
      int dayYear=Date77.dayOfYear(hit.time);
      if (dayYear>day) {
        ips=new HashSet<String>();
        day=dayYear;
        res+=
"<tr><td><b>"+Date77.date(hit.time)+th+"referer"+th+"ip"+th+"hits</b></td></tr>\n";
      }
      String ip=hit.user.ip;
      if (!ips.contains(ip)) {
        ips.add(ip);
        res+=
"<tr><td>"+Date77.time(hit.time)+td+SiteStatsUtil.getFirstReferer(ip)+td+ip+td+hitCount(ip, day, hits)+"</td></tr>\n";
      }
    }

    File77.create(dir+"/by_days.html", res+"</table><p>\n"+
"  </body>\n</html>\n");
  }

  private synchronized static final int hitCount(String ip, int dayYear,
      List<Hit> sortedHits) {
    int count=0;
    for (Hit hit: sortedHits) {
      if (hit.user.ip.equals(ip) && Date77.dayOfYear(hit.time)==dayYear) count++;
    }
    return count;
  }

  private static final String getBrowser(String userAgent) {
    if (userAgent.contains("MSIE 7.0")) return "IE 7.0";
    if (userAgent.contains("MSIE 6.0")) return "IE 6.0";
    if (userAgent.contains("MSIE 5.5")) return "IE 5.5";
    if (userAgent.contains("Opera/9")&&userAgent.contains("Windows")) {
      return "Opera/9 Windows";
    }
    if (userAgent.contains("Googlebot")) return "Googlebot";
    if (userAgent.contains("Mozilla/2.0 (compatible; Ask Jeeves/Teoma")) {
      return "Mozilla/2.0 (Ask Jeeves/Teoma)";
    }
    if (userAgent.contains("Mozilla/5.0") && userAgent.contains("Linux")) {
      return "Mozilla/5.0 Linux";
    }
    if (userAgent.contains("Mozilla/5.0") && userAgent.contains("Macintosh")) {
      return "Mozilla/5.0 Macintosh";
    }
    if (userAgent.contains("Mozilla/5.0") && userAgent.contains("Windows")) {
      return "Mozilla/5.0 Windows";
    }
    if (userAgent.contains("Windows") && userAgent.contains("Firefox/2")) {
      return "Windows Firefox/2";
    }
    return userAgent;
  }

  private static final String getReferer(String ref) {
    String btr="http:"+P.bs+"btrgame.com";
    return ref.startsWith(btr)?ref.substring(btr.length()):ref;
  }

  public static final String getMyUrl(HttpServletRequest r) {
    String url=r.getRequestURL().toString();
    if (url.startsWith("http:"+P.bs+"btrgame")) {
      return r.getServletPath();
    } else {
      return url.substring(("http:"+P.bs).length(), url.indexOf('.'))+" "+
          r.getServletPath();
    }
  }
}
